/**
 * The program(s) herein may be used  freely
 * for personal use only. It may not be sold, 
 * rented, leased, sublicensed  or  modified 
 * without permission of  the  author.  
 * 
 *           www.carlosbcruz.com
 *           
 * This program is provided "AS IS"  without 
 * warranty of any kind. In no event I  will
 * be liable  for  any  direct  or  indirect 
 * damage.
 */

package com.carlosbcruz.bruteforce;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Vector;

/**
 * Implement the brute force algorithm
 * 
 * @author Carlos Fernando Bella Cruz - <a href="mailto:pessoal@carlosbcruz.com">pessoal@carlosbcruz.com</a>
 */
public class BruteForce implements Runnable {

    private Thread runner = null;

    private Problem problem = null;
    private boolean solved = false;

    private int numCall = 0;

    private boolean go = false;

    private Vector solution = new Vector();

    /**
     * Brute Force algorith solves a problem
     * 
     * @param problem
     */
    public BruteForce(Problem problem) {

        this.problem = problem;

    }

    /**
     * Is the command ready to be executed ?
     * 
     * @return
     */
    public boolean isReadyToExecuteCall() {

        return (problem != null);

    }

    /**
     * Solve a problem through brute force
     * @param problem the problem
     * @return true if the problem has been solved
     */
    private boolean solve(Problem problem) {

        Vector possib = problem.getPossibilities();

        Vector newProblems = new Vector();

        numCall++;

        if (go == false)
            return false;

        if (numCall % 100000 == 0)
            System.out.println(numCall + " possibilities searched.");

        for (int i = 0; i < possib.size(); i++) {

            Possibility p = (Possibility)possib.elementAt(i);

            Problem newProb = null;

            try {
                newProb = (Problem)problem.clone();
            }
            catch(CloneNotSupportedException e) {
                System.out.println("Clone Error 1 !");
            }

            newProb.apply(p);

            if (newProb.isSolved()) {
                try {
                    solution.addElement(newProb.clone());
                }
                catch(CloneNotSupportedException e) {
                    System.out.println("Clone Error 2 !");
                }
                System.out.println("A solution has been found");
                return true;
            }
            newProblems.addElement(newProb);

        }

        for (int i = 0; i < newProblems.size(); i++) {

            if (solve((Problem)newProblems.elementAt(i))) {
                try {
                    solution.addElement(((Problem)newProblems.elementAt(i))
                            .clone());
                }
                catch(CloneNotSupportedException e) {
                    System.out.println("Clone Error 2 !");
                }
                return true;
            }
        }

        return false;
    }

    /* (non-Javadoc)
     * @see java.lang.Runnable#run()
     */
    public void run() {

        solved = solve(problem);

        if (solved) {
            System.out
                    .println("\n A solution has been written to Solution.txt file.");

            try {

                File fOut = new File("Solution.txt");
                FileWriter fw = new FileWriter(fOut);
                BufferedWriter out = new BufferedWriter(fw);

                solution.addElement(problem);
                out.write("SOLUTION:\n");
                for (int i = solution.size() - 1; i >= 0; i--) {
                    Problem p = (Problem)solution.elementAt(i);
                    out.write("\n");
                    out.write(p.toString());
                }

                out.close();

            }
            catch(IOException e) {

                System.out.println("Error --" + e.toString());

            }
            catch(SecurityException se) {

                System.out.println("Error --" + se.toString());

            }
        } else {
            System.out.println("There is no solution.");
        }
    }

    /**
     * Start solving the problem
     */
    public void execute() {

        if (go)
            return;

        runner = new Thread(this);
        runner.start();
        go = true;
    }

    //
    // Stop solving the problem
    //
    public void stop() {

        System.exit(0);
    }

    public Vector getSolution() {

        return solution;
    }

}