/**
 * The program(s) herein may be used  freely
 * for personal use only. It may not be sold, 
 * rented, leased, sublicensed  or  modified 
 * without permission of  the  author.  
 * 
 *           www.carlosbcruz.com
 *           
 * This program is provided "AS IS"  without 
 * warranty of any kind. In no event I  will
 * be liable  for  any  direct  or  indirect 
 * damage.
 */

package com.carlosbcruz.pegsolitaire;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JPanel;

/**
 * Implement the gui of the application
 * 
 * @author Carlos Fernando Bella Cruz - <a href="mailto:pessoal@carlosbcruz.com">pessoal@carlosbcruz.com</a>
 */
public class OnlyOneGUI extends JFrame {

    /**
     * First version 
     */
    private static final long serialVersionUID = 1L;
    
    private Vector listenersStart = new Vector();
    private Vector listenersStop = new Vector();

    private JButton start = new JButton("Start");
    private JButton stop = new JButton("Stop");

    /**
     * Default constructor 
     */
    public OnlyOneGUI() {

        super("Peg Solitaire Problem");

        start.addActionListener(new OnlyOneGUIActionListenerStart());
        stop.addActionListener(new OnlyOneGUIActionListenerStop());

        JPanel pane = new JPanel();

        pane.add(start);
        pane.add(stop);

        setContentPane(pane);

        addWindowListener(new OnlyOneGUIAdapter());
        pack();
        setVisible(true);

    }

    /**
     * Add a listener to the start button
     * @param listener
     */
    public synchronized void addOnlyOneGUIStartListener(
        OnlyOneGUIListener listener) {

        listenersStart.addElement(listener);
    }

    /**
     * Remove a listener to the start button
     * @param listener
     */
    public synchronized void removeOnlyOneGUIStartListener(
        OnlyOneGUIListener listener) {

        listenersStart.removeElement(listener);
    }

    /**
     * Add a listener to the stop button
     * @param listener
     */
    public synchronized void addOnlyOneGUIStopListener(
        OnlyOneGUIListener listener) {

        listenersStop.addElement(listener);
    }

    /**
     * Remove a listener to the stop button
     * @param listener
     */
    public synchronized void removeOnlyOneGUIStopListener(
        OnlyOneGUIListener listener) {

        listenersStop.removeElement(listener);
    }

    /**
     * Notify that the button start has been pressed 
     */
    protected void notifyOnlyOneGUIStart() {

        Vector l;

        CommandOnlyOneEvent e = new CommandOnlyOneEvent(this);

        synchronized (this) {
            l = (Vector)listenersStart.clone();
        }

        for (int i = 0; i < l.size(); i++) {
            ((OnlyOneGUIListener)l.elementAt(i)).start(e);
        }
    }

    /**
     * Notify that the button stop has been pressed 
     */
    protected void notifyOnlyOneGUIStop() {

        Vector l;

        CommandOnlyOneEvent e = new CommandOnlyOneEvent(this);

        synchronized (this) {
            l = (Vector)listenersStop.clone();
        }

        for (int i = 0; i < l.size(); i++) {
            ((OnlyOneGUIListener)l.elementAt(i)).stop(e);
        }
    }

    /**
     * Listen to window events
     * 
     * @author Carlos Fernando Bella Cruz - <a href="mailto:pessoal@carlosbcruz.com">pessoal@carlosbcruz.com</a>
     */
    class OnlyOneGUIAdapter extends WindowAdapter {

        /* (non-Javadoc)
         * @see java.awt.event.WindowAdapter#windowClosing(java.awt.event.WindowEvent)
         */
        public void windowClosing(WindowEvent e) {

            System.exit(0);
        }

    };

    /**
     * Listen to start button
     * 
     * @author Carlos Fernando Bella Cruz - <a href="mailto:pessoal@carlosbcruz.com">pessoal@carlosbcruz.com</a>
     */
    class OnlyOneGUIActionListenerStart implements ActionListener {

        /* (non-Javadoc)
         * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
         */
        public void actionPerformed(ActionEvent evt) {

            notifyOnlyOneGUIStart();
        }
    };

    /**
     * Listen to stop button
     * 
     * @author Carlos Fernando Bella Cruz - <a href="mailto:pessoal@carlosbcruz.com">pessoal@carlosbcruz.com</a>
     */
    class OnlyOneGUIActionListenerStop implements ActionListener {

        /* (non-Javadoc)
         * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
         */
        public void actionPerformed(ActionEvent evt) {

            notifyOnlyOneGUIStop();
        }
    };
}