/**
 * The program(s) herein may be used  freely
 * for personal use only. It may not be sold, 
 * rented, leased, sublicensed  or  modified 
 * without permission of  the  author.  
 * 
 *           www.carlosbcruz.com
 *           
 * This program is provided "AS IS"  without 
 * warranty of any kind. In no event I  will
 * be liable  for  any  direct  or  indirect 
 * damage.
 */
package com.carlosbcruz.pegsolitaire;

import com.carlosbcruz.bruteforce.*;

/**
 * Represent one possibility of movement.
 * 
 * @author Carlos Fernando Bella Cruz - <a href="mailto:pessoal@carlosbcruz.com">pessoal@carlosbcruz.com</a>
 */
public class OnlyOnePossibility implements Possibility {

    private boolean applied;
    private int x_from, y_from, x_to, y_to, x_out, y_out;

    /**
     * Constructor that represent a movement. An element can be moved
     * from x,y coordinates to x,y coordinates an removing a x,y element 
     * from the board.
     * @param x_from x coordinate of the element to be moved
     * @param y_from y coordinate of the element to be moved
     * @param x_to x coordinate of the destination of the element
     * @param y_to y coordinate of the destination of the element
     * @param x_out x coordinate of the element to be removed
     * @param y_out y coordinate of the element to be removed
     */
    public OnlyOnePossibility(int x_from, int y_from, int x_to, int y_to,
        int x_out, int y_out) {

        this.x_from = x_from;
        this.y_from = y_from;
        this.x_to = x_to;
        this.y_to = y_to;
        this.x_out = x_out;
        this.y_out = y_out;
    }

    /**
     * @return x coordinate of the element to be moved
     */
    public int getXfrom() {

        return x_from;
    }

    /**
     * @return y coordinate of the element to be moved
     */
    public int getYfrom() {

        return y_from;
    }

    /**
     * @return x coordinate of the destination of the element
     */
    public int getXto() {

        return x_to;
    }

    /**
     * @return y coordinate of the destination of the element
     */
    public int getYto() {

        return y_to;
    }

    /**
     * @return x coordinate of the element to be removed
     */
    public int getXout() {

        return x_out;
    }

    /**
     * @return y coordinate of the element to be removed
     */
    public int getYout() {

        return y_out;
    }

    /* (non-Javadoc)
     * @see bruteforce.Possibility#applied()
     */
    public void applied() {

        applied = true;
    }

    /* (non-Javadoc)
     * @see bruteforce.Possibility#wasApplied()
     */
    public boolean wasApplied() {

        return applied;
    }

    /* (non-Javadoc)
     * @see java.lang.Object#toString()
     */
    public String toString() {

        String out = new String();
        out = "X From: " + (x_from + 1) + " Y From: " + (y_from + 1)
            + " X To: " + (x_to + 1) + " Y To: " + (y_to + 1) + " X Out: "
            + (x_out + 1) + " Y Out: " + (y_out + 1) + "\n";

        return out;
    }
}